# Name :        Balustrade
# Description : Creates balustrade with desired dimmensions
# Author :      Tomasz MAREK, modified D. Bur
# Usage :       1.Menu - Draw/Balustrade
#		2.Give dimmensions and Ok.			
# Date :        14.Jul.2oo5, December 2005
# Type :        Architecture tool
# History:        
#                
#                 1.o  (14.July.2oo5) - first version, version 1.1 & 1.2. - November 2005

require 'sketchup.rb'


def lines2balustrade
model = Sketchup.active_model
ss = model.selection
notanedge = 0

if ss.empty? 
  UI.messagebox("No selection. Select line(s) please.")
  return nil
end

ss.each do |e|
  if( e.typename != "Edge")
    ss.remove e
    notanedge = notanedge + 1
  end
end

if (notanedge > 0)
  UI.messagebox( notanedge.to_s + " objects are not lines and will be ignored.")
end

# default_values
$handrail_section = "Rectangular" if not $handrail_section
$post_section = "Rectangular" if not $post_section
$panel_section = "None" if not $panel_section

$height = 1000.mm if not $height
$jump = 500.mm if not $jump
$post_width = 50.mm if not $post_width
$post_depth = 50.mm if not $post_depth
$post_diam = 50.mm if not $post_diam
$handrail_height= 40.mm if not $handrail_height
$handrail_depth = 60.mm if not $handrail_depth
$handrail_diam = 60.mm if not $handrail_diam
$panel_start_height = 200.mm if not $panel_start_height 
$panel_height = 500.mm if not $panel_height
$panel_thickness = 10.mm if not $panel_thickness

components_list = (collect_components)
components_list.sort!
components_list.reverse!

inputbox1 = components_list.join("|") + "|None|Circular|Rectangular"
inputbox2 = components_list.join("|") + "|None|Circular|Rectangular"
inputbox3 = components_list.join("|") + "|None|Rectangular"
array_of_dropdowns = [inputbox1, inputbox2,inputbox3]
  
prompts = ["Handrail Section:  ", "Posts Section:  ", "Panels:  ", "Balustrade Height:  ", "Posts spacing:  "]
values = [$handrail_section, $post_section, $panel_section, $height, $jump]
results = inputbox prompts, values, array_of_dropdowns, "Balustrade elements"
return if not results
$handrail_section, $post_section, $panel_section, $height, $jump = results

# Construct the prompts dialog and values
prompts = []
$values = []
$variables = []
case $handrail_section
  when "Rectangular"
    prompts.push("Handrail: width ", "Handrail: height ")
    $values.push($handrail_depth, $handrail_height)
    $variables.push("$handrail_depth", "$handrail_height")
    $handrail_diam = nil
  when "Circular"
    prompts.push("Handrail: diameter ")
    $values.push($handrail_diam)
    $variables.push("$handrail_diam")
    $handrail_height = nil
    $handrail_depth = nil
  when "None"
    $handrail_diam = nil
    $handrail_height = nil
    $handrail_depth = nil
  else
    $handrail_diam = nil
    $handrail_height = nil
    $handrail_depth = nil
end
case $post_section
  when "Rectangular"
    prompts.push("Post: width ", "Post: depth ")
    $values.push($post_width, $post_depth)
    $variables.push("$post_width", "$post_depth")
    $post_diam = nil
  when "Circular"
    prompts.push("Post: diameter ")
    $values.push($post_diam)
    $variables.push("$post_diam")
    $post_width = nil
    $post_depth = nil
  when "none"
    $post_diam = nil
    $post_width = nil
    $post_depth = nil
  else
    $post_diam = nil
    $post_width = nil
    $post_depth = nil
end
case $panel_section
  when "Rectangular"
    prompts.push("Panel: height above ground ", "Panel: height ", "Panel: thickness ")
    $values.push($panel_start_height, $panel_height, $panel_thickness)
    $variables.push("$panel_start_height", "$panel_height", "$panel_thickness")
  when "None"
    $panel_start_height = nil
    $panel_height = nil
    $panel_thickness = nil
  else
    prompts.push("Pannel: height above ground ")
    $values.push($panel_start_height)
    $panel_height = nil
    $panel_thickness = nil
end

# User inputs
if prompts!= []
  results = inputbox prompts, $values, "Balustrade Dimensions"
  # Eval variables
  i = 0
  $variables.each do |v|
    case v
      when "$post_width"
        $post_width = results[i]
      when "$post_depth"
        $post_depth = results[i]
      when "$post_diam"
        $post_diam = results[i]
      when "$handrail_height"
        $handrail_height = results[i]
      when "$handrail_depth"
        $handrail_depth =results[i]
      when "$handrail_diam"
        $handrail_diam = results[i]
      when "$panel_start_height"
        $panel_start_height = results[i]
      when "$panel_height"
        $panel_height = results[i]
      when "$panel_thickness"
        $panel_thickness = results[i]
    end
    i = i + 1
  end
end


ss.each do |element|
  $line = element
  Balustrade.new.create_balus
end

end

# Collect components and return an array of ComponentsDefinitions names
def collect_components
model=Sketchup.active_model
defs = model.definitions
ret_defs = []
defs.each do |c|
  if not c.name["Image"] and not c.name["Group#"]
    ret_defs = ret_defs.push(c.name)
  end
end
return ret_defs
end

def tolerance( n1, n2, interval)
if (n1.between?((n2 - interval), (n2 + interval)))
  return true
  else
  return false
end
end

# Return angle between v1 and v2, 0->2PI
def angle_between_vec( v2 )
v1 = Geom::Vector3d.new(1, 0, 0)
v2x = v2[0]
v2y = v2[1]
v2z = v2[2]
x = 1.0
y = 1.0

x = (tolerance(v2x, 0.0, 0.00001))
y = (tolerance(v2y, 0.0, 0.00001))

# 0,90,180,270
if (x == true) and ( y == false )
  if ( v2y > 0.0 ) and ( y == false )
    a = (Math::PI / 2.0)
  end
  if ( v2y < 0.0 ) and ( y == false )
    a = (3 * (Math::PI / 2.0))
  end
end

if ( x == false ) and (y == true)
  if ( v2x > 0.0 ) and (y == true)
    a = 0.0
  end
  if ( v2x < 0.0 ) and (y == true)
    a = Math::PI
  end
end

# Other angles
if ( v2x > 0.0 ) and ( y == false ) and ( x == false )
    if ( v2y > 0.0 ) and ( y == false )
      a = v1.angle_between( v2 )
    end
    if ( v2y < 0.0 ) and ( y == false )
      a = Math::PI - v1.angle_between( v2 )
    end
end

if ( v2x < 0.0 ) and ( y == false ) and ( x == false )
    if ( v2y > 0.0 ) and ( y == false )
      a = v1.angle_between( v2 )
    end
    if ( v2y < 0.0 ) and ( y == false )
      a = Math::PI - v1.angle_between( v2 )
    end
end
return a
end

# Returns an array of 4 points to draw a vertical rectangle (l x w), p is the center of rectangle, v is the normal to rectangle
def vertical_section( p, v, l, w)
vx = v.axes[0]
vmx = v.axes[0].reverse
vx.length = l/2.0
vmx.length = l/2.0
vy = v.axes[1]
vmy = v.axes[1].reverse
vy.length = w/2.0
vmy.length = w/2.0
tx = Geom::Transformation.new(vx)
tmx = Geom::Transformation.new(vmx)
ty = Geom::Transformation.new(vy)
tmy = Geom::Transformation.new(vmy)
pt = p.transform(tmy)###TIG'd
p1 = pt.transform(tmx)###TIG'd
p2 = p1.transform(ty)###TIG'd
p2 = p2.transform(ty)###TIG'd
p3 = p2.transform(tx)###TIG'd
p3 = p3.transform(tx)###TIG'd
p4 = p3.transform(tmy)###TIG'd
p4 = p4.transform(tmy)###TIG'd
return [p1, p2, p3, p4]
end

# Returns an array of 4 points to draw a horizontal rectangle (l x w), p is the center of rectangle, v is the xaxis of rectangle
def horizontal_section( p, v, l, w)
#model = Sketchup.active_model
#entities = model.active_entities
vx = v
vmx = v.reverse
vx.length = l/2.0
vmx.length = l/2.0
rot =Geom::Transformation.rotation(p, Geom::Vector3d.new(0, 0, 1), Math::PI/2.0)###TIG'd
vy = vx.transform(rot)###TIG'd
vmy = vy.reverse
vy.length = w/2.0
vmy.length = w/2.0
tx = Geom::Transformation.new(vx)
tmx = Geom::Transformation.new(vmx)
ty = Geom::Transformation.new(vy)
tmy = Geom::Transformation.new(vmy)
pt = p.transform(tmy)###TIG'd
p1 = pt.transform(tmx)###TIG'd
p2 = p1.transform(ty)###TIG'd
p2 = p2.transform(ty)###TIG'd
p3 = p2.transform(tx)###TIG'd
p3 = p3.transform(tx)###TIG'd
p4 = p3.transform(tmy)###TIG'd
p4 = p4.transform(tmy)###TIG'd
return [p1, p2, p3, p4]

end

class Balustrade

def create_balus

model = Sketchup.active_model
entities = model.active_entities
definitions = model.definitions

start_point = $line.start.position
end_point = $line.end.position
length = $line.length
line = $line.line
vec = line[1]
# ----

# ----
model.start_operation "Creer Balustrade"
entities = model.active_entities
group = entities.add_group
entities = group.entities

# Posts heights and length of panels
if $handrail_depth and $post_depth
  post_height = $height - $handrail_height
  length_panel = $jump - $post_width
end
if $handrail_depth and $post_diam
  post_height = $height - $handrail_height
  length_panel = $jump - $post_diam
end
if $handrail_diam and $post_depth
  post_height = $height - ($handrail_diam / 2.0)
  length_panel = $jump - $post_width
end
if $handrail_diam and $post_diam
  post_height = $height - ($handrail_diam / 2.0)
  length_panel = $jump - $post_diam
end
if !$handrail_depth and !$handrail_diam
  post_height = $height - 5.cm
end

# Handrail
  if $handrail_height
    # Rectangular section
    handrail_start_point = [start_point.x, start_point.y, start_point.z + $height - ($handrail_height / 2.0)]
    pts = vertical_section(handrail_start_point, vec, $handrail_depth, $handrail_height)
    handrail_face = entities.add_face(pts)###TIG'd
    handrail_face.pushpull(-length)###TIG'd
  end
  if $handrail_diam
    # Circular section
    handrail_start_point = [start_point.x, start_point.y, start_point.z + $height - ($handrail_diam / 2.0)]
    section = entities.add_circle(handrail_start_point, vec, ($handrail_diam/2.0), 10)###TIG'd
    circle = entities.add_face(section)###TIG'd
    circle.pushpull(length)###TIG'd
  end
if !$handrail_height and !$handrail_diam 
  # Component section
  handrail_start_point = [start_point.x, start_point.y, start_point.z + $height]
  tt = Geom::Transformation.new(handrail_start_point)###TIG'd
  comp_start_point = handrail_start_point
  rot_angle = Geom::Vector3d.new(1, 0, 0).angle_between(vec)###TIG'd
  tr = Geom::Transformation.rotation(start_point, [0,0,1], rot_angle)###TIG'd
  post_component = entities.add_instance(definitions[$handrail_section], tt)###TIG'd
  post_component.transform!(tr)###TIG'd
end

#Posts
  if $post_depth
    # First rectangular post
    v = vec
    v.length = $post_width / 2.0
    t = Geom::Transformation.new(v)
    post_start_point = start_point.transform(t)###TIG'd
    pts = horizontal_section(post_start_point, vec, $post_width, $post_depth)
    base = entities.add_face(pts)###TIG'd
    base.reverse!.pushpull(post_height)###TIG'd
    length_done = $post_width / 2.0
    # Intermediate rectangular posts
    if length>$jump
      while length_done < length - (($post_width * 3) / 2.0) - $jump
        vec.length = $jump
        tj = Geom::Transformation.new(vec)
        post_start_point.transform!(tj)###TIG'd
        pts = horizontal_section(post_start_point, vec, $post_width, $post_depth)
        base = entities.add_face(pts)###TIG'd
        base.reverse!.pushpull(post_height)###TIG'd
        length_done = length_done + $jump
      end
    end
    # Last rectangular post
    v = vec.reverse
    v.length = $post_width / 2.0
    t = Geom::Transformation.new(v)
    post_start_point = end_point.transform(t)###TIG'd
    pts = horizontal_section(post_start_point, vec, $post_width, $post_depth)
    base = entities.add_face(pts)###TIG'd
    base.reverse!.pushpull(post_height)###TIG'd
    length_done = $post_width / 2.0
  end
  
  if $post_diam
  # First circular post
    v = vec
    v.length = $post_diam / 2.0
    t = Geom::Transformation.new(v)
    post_start_point = start_point.transform(t)###TIG'd
    section = entities.add_circle(post_start_point, vec.axes[1], ($post_diam/2.0), 10)###TIG'd
    circle = entities.add_face(section)###TIG'd
    if( circle.normal.dot(Z_AXIS) > 0 )
      circle.pushpull(post_height)###TIG'd
      else
      circle.reverse!.pushpull(post_height)###TIG'd
    end
    length_done = $post_diam / 2.0
      
  # Intermediate circular posts
    if length>$jump
      while length_done < length - (($post_diam * 3) / 2.0) - $jump
        vec.length = $jump
        tj = Geom::Transformation.new(vec)
        post_start_point.transform!(tj)###TIG'd
        section = entities.add_circle(post_start_point, vec.axes[1], ($post_diam/2.0), 10)###TIG'd
        circle = entities.add_face(section)###TIG'd
        if( circle.normal.dot(Z_AXIS) > 0 )
          circle.pushpull($height-($post_diam/2.0))###TIG'd
          else
          circle.reverse!.pushpull(post_height)###TIG'd
        end
        length_done = length_done + $jump
      end
    end    
  # Last circular post
    v = vec.reverse
    v.length = $post_diam / 2.0
    t = Geom::Transformation.new(v)
    post_start_point = end_point.transform(t)###TIG'd
    section = entities.add_circle(post_start_point, vec.axes[1], ($post_diam/2.0), 10)###TIG'd
    circle = entities.add_face(section)###TIG'd
    if( circle.normal.dot(Z_AXIS) > 0 )
      circle.pushpull($height-($post_diam/2.0))###TIG'd
      else
      circle.reverse!.pushpull(post_height)###TIG'd
    end
    length_done = $post_diam / 2.0
  end
  
  if ($post_section != "Rectangular") and ($post_section != "Circular")
    #First component post
    tt = Geom::Transformation.new(start_point)###TIG'd
    comp_start_point = start_point
    rot_angle = Geom::Vector3d.new(1, 0, 0).angle_between(vec)###TIG'd
    tr = Geom::Transformation.rotation(start_point, [0,0,1], rot_angle)###TIG'd
    post_component = entities.add_instance(definitions[$post_section], tt)###TIG'd
    post_component.transform!(tr)###TIG'd
    length_done = 0.cm
    
    # Intermediate component posts
    if length>$jump
      while length_done < length - $jump
        vec.length = length_done + $jump
        comp_start_point = start_point.offset vec
        tt = Geom::Transformation.new(comp_start_point)###TIG'd
        rot_angle = Geom::Vector3d.new(1, 0, 0).angle_between(vec)###TIG'd
        tr = Geom::Transformation.rotation(comp_start_point, [0,0,1], rot_angle)###TIG'd
        post_component = entities.add_instance(definitions[$post_section], tt)###TIG'd
        post_component.transform!(tr)###TIG'd
        length_done = length_done + $jump
      end
    end
    
    #Last component post
    vec.length = length
    comp_start_point = start_point.offset vec
    tt = Geom::Transformation.new(comp_start_point)###TIG'd
    rot_angle = Geom::Vector3d.new(1, 0, 0).angle_between(vec)###TIG'd
    tr = Geom::Transformation.rotation(comp_start_point, [0,0,1], rot_angle)###TIG'd
    post_component = entities.add_instance(definitions[$post_section], tt)###TIG'd
    post_component.transform!(tr)###TIG'd
        
  end # of component posts

# Rectangular panels
if ($panel_section == "Rectangular")
  num_panels = (length/$jump).floor.to_i
  v1 = vec
  length_done = 0.0
  
  if $post_width and $post_depth
    # Rect posts
    v1.length = ($post_width / 2.0) + ($jump / 2.0)
    panel_width = $jump - $post_width
    half_post = $post_width / 2.0
  end
  if $post_diam and !$post_width and !$post_depth
    # Circ posts
    v1.length = ($post_diam / 2.0) + ($jump / 2.0)
    panel_width = $jump - $post_diam
    half_post = $post_diam / 2.0
  end
  if !$post_diam and !$post_width and !$post_depth
    # Component
    v1.length = ($jump / 2.0) + 5.cm
    panel_width = $jump - 10.cm
    half_post = 5.cm
  end
  
  t1 = Geom::Transformation.translation(v1)
  tup = Geom::Transformation.new(Geom::Vector3d.new(0, 0, $panel_start_height))###TIG'd
  
  # First panel
  panel_start_point = start_point.transform(t1)###TIG'd
  panel_start_point = panel_start_point.transform(tup)###TIG'd
  pts = horizontal_section(panel_start_point, vec, panel_width, $panel_thickness)
  panel = entities.add_face(pts)###TIG'd
  if( panel.normal.dot(Z_AXIS) > 0 )
    panel.pushpull($panel_height)###TIG'd
  else
    panel.reverse!.pushpull($panel_height)###TIG'd
  end
  length_done = half_post + ($jump / 2.0)
  
  # Other panels
  vt = vec
  vt.length = $jump
  t2 = Geom::Transformation.translation(vt)
  0.upto (num_panels - 2) do |i|
    panel_start_point= panel_start_point.transform(t2)###TIG'd
    pts = horizontal_section(panel_start_point, vec, panel_width, $panel_thickness)
    panel = entities.add_face(pts)###TIG'd
    if( panel.normal.dot(Z_AXIS) > 0 )
      panel.pushpull($panel_height)###TIG'd
    else
      panel.reverse!.pushpull($panel_height)###TIG'd
    end
    length_done = length_done + $jump
  end
  
  # Last panel
  vtt = vec
  panel_width = (length - length_done) - (($jump / 2.0) + (half_post * 3))
  vtt.length = ($jump / 2.0) + half_post + (panel_width / 2.0) 
  t3 = Geom::Transformation.translation(vtt)
  panel_start_point= panel_start_point.transform(t3)###TIG'd
  pts = horizontal_section(panel_start_point, vec, panel_width, $panel_thickness)
  panel = entities.add_face(pts)
  if( panel.normal.dot(Z_AXIS) > 0 )
    panel.pushpull($panel_height)###TIG'd
  else  
    panel.reverse!.pushpull($panel_height)###TIG'd
  end
  length_done = length_done + $jump

end #draw rectangular panels

if ($panel_section != "Rectangular") and ($panel_section != "Aucun")
  num_panels = (length/$jump).floor.to_i
  v1 = vec
  length_done = 0.cm
  panel_width = $jump / 2.0
  
  if $post_width and $post_depth
    # Rect posts
    v1.length = ($post_width / 2.0) + ($jump / 2.0)
    half_post = $post_width / 2.0
  end
  if $post_diam and !$post_width and !$post_depth
    # Circ posts
    v1.length = ($post_diam / 2.0) + ($jump / 2.0)
    half_post = $post_diam / 2.0
  end
  if !$post_diam and !$post_width and !$post_depth
    # Component
    v1.length = ($jump / 2.0)
    panel_width = $jump
    half_post = 0.cm
  end
  
  tup = Geom::Transformation.new(Geom::Vector3d.new(0, 0, $panel_start_height))###TIG'd
  tjump = Geom::Transformation.translation(v1)
  
  comp_start_point = start_point.transform(tup)###TIG'd
  comp_start_point.transform!(tjump)###TIG'd
 
 # First component panel
  tt = Geom::Transformation.translation(comp_start_point)
  rot = angle_between_vec(vec)###TIG'd
  tr = Geom::Transformation.rotation(comp_start_point, [0,0,1], rot)###TIG'd
  post_component = entities.add_instance(definitions[$panel_section], tt)###TIG'd
  post_component.transform!(tr)###TIG'd
  length_done = $jump / 2.0
  
# Other components panels
  vt = vec
  vt.length = $jump
  tjump = Geom::Transformation.translation(vt)
  0.upto (num_panels - 1) do |i|
    comp_start_point = comp_start_point.transform!(tjump)###TIG'd
    tt = Geom::Transformation.translation(comp_start_point)
    post_component = entities.add_instance(definitions[$panel_section], tt)###TIG'd
    rot = angle_between_vec(vec)###TIG'd
    tr = Geom::Transformation.rotation(comp_start_point, [0,0,1], rot)###TIG'd
    post_component.transform!(tr)###TIG'd
    length_done = length_done + $jump
  end
  
end #draw components panels

  model.commit_operation
end 

end #class Balustrade


if( not file_loaded?("balustrade_components.rb") )
    UI.menu("Plugins").add_item("Balustrade with components") { lines2balustrade }

end


file_loaded("balustrade_components.rb")



